/*
 * main.c
 *
 *  Created on: Mar 8, 2018
 *      Author: dadan
 */

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include <avr/power.h>

#include "buttons.h"
#include "time_font.h"
#include "fonts.h"
#include "spi.h"
#include "epd_1_54.h"
#include "canvas.h"
#include "basic_apps.h"
#include "timers.h"

// #define SECONDS_PROGRESSBAR
// #define REFRESH_1HZ


// debug and testing
// #define TEST_TEXT
#define CLEAR_DISPLAY_ON_START


volatile time_t time;
volatile uint8_t display_refresh_needed;
volatile uint8_t sw_timer[SW_TIMER_COUNT];

image_buffer_t image_buffer;

// ISR(TIMER1_COMPA_vect);
ISR(TIMER2_OVF_vect);

int main(void)
{
	// -- Initialize e-paper display --
	spi_init();
	epd_reset();
	epd_init_full(DISPLAY_TEMPERTURE);
	//epd_init_partial(DISPLAY_TEMPERTURE);

	// reset software timers
	for (uint8_t i = 0; i < SW_TIMER_COUNT; ++i)
	{
		sw_timer[i] = 0;
	}

	// vibrate
	DDRD |= (1<<DDD5) | (1<<DDD6);
	PORTD |= (1<<PORTD5);	// vibrate
	PORTD |= (1<<PORTD6);	// enable backlight
	_delay_ms(500);
	PORTD &= ~(1<<PORTD5);
	PORTD &= ~(1<<PORTD6);	// disable backlight


    // clear image on display
	// todo: improve this
#ifdef CLEAR_DISPLAY_ON_START
    epd_clear_frame_memory(COLOR_WHITE);		// set - white; not set - black
    epd_display_frame();
    epd_clear_frame_memory(COLOR_WHITE);
    epd_display_frame();
#endif

    // initialize Timer/Counter 2 in asynchronous mode with external 32.768 kHz crystal
    ASSR = (1<<AS2);				// enable asynchronous operation - this bit must be set before TCCR2x
    TCCR2A = 0;						// WGM mode 0 - normal operation
    TCCR2B = (1<<CS22) | (1<<CS20);	// prescaler /128 - one overflow per second
    TIMSK2 = (1<<TOIE2);			// enable overflow interrupt

    // enable pin change interrupt for buttons
    PCICR = (1<<PCIE2) | (1<<PCIE1);
    PCMSK2 |= (1<<PCINT19) | (1<<PCINT18);
    PCMSK1 |= (1<<PCINT10) | (1<<PCINT9);

	// init time
	time.hours = 12;
	time.minutes = 34;
	time.seconds = 0;

	display_refresh_needed = 1;

	set_sleep_mode(SLEEP_MODE_PWR_SAVE);

	power_adc_disable();
	power_timer0_disable();
	power_timer1_disable();
	power_twi_disable();
	power_usart0_disable();

	// Enable pull-up on all unused pins to define voltage on pin (decrease opwer consumption)
//	PORTB |= (1<<PORTB6) | (1<<PORTB7);
//	PORTC |= 0xFF;
//	PORTD |= (1<<PORTD6) | (1<<PORTD5) | (1<<PORTD4) | (1<<PORTD3) | (1<<PORTD2) | (1<<PORTD1) | (1<<PORTD0);

	button_init();		// initialize buttons

	sei();		// enable global interrupts


	while (1)
	{
    	epd_reset();			// todo: wasting with power in delay loops
    	epd_init_partial(DISPLAY_TEMPERTURE);
    	//epd_init_full(DISPLAY_TEMPERTURE);

#ifdef EPD_SIZE_1_54

	    canvas_display_from_flash(&image_buffer, &time_font[time.hours / 10], 48, 152, 1);	// fist digit
	    canvas_display_from_flash(&image_buffer, &time_font[time.hours % 10], 48, 104, 1);	// second digit

	    canvas_display_from_flash(&image_buffer, &time_font[10], 48, 96, 1);	// delimiter

	    canvas_display_from_flash(&image_buffer, &time_font[time.minutes / 10], 48, 48, 1);	// third digit
	    canvas_display_from_flash(&image_buffer, &time_font[time.minutes % 10], 48, 0, 1);		// fourth digit
#endif

#ifdef EPD_SIZE_2_13

	    canvas_display_from_flash(&image_buffer, &time_font[time.hours / 10], 8, 176, 1);	// fist digit
	    canvas_display_from_flash(&image_buffer, &time_font[time.hours % 10], 8, 128, 1);	// second digit

	    canvas_display_from_flash(&image_buffer, &time_font[10], 8, 120, 1);	// delimiter

	    canvas_display_from_flash(&image_buffer, &time_font[time.minutes / 10], 8, 72, 1);	// third digit
	    canvas_display_from_flash(&image_buffer, &time_font[time.minutes % 10], 8, 24, 1);		// fourth digit
#endif


	    // show progressbar with seconds
		#ifdef SECONDS_PROGRESSBAR
	    image_buffer.width = 8;
	    image_buffer.height = 200;

	    for (uint16_t i = 0; i < IMAGE_BUFFER_SIZE; ++i)
	    {
	    	image_buffer.buffer[i] = COLOR_WHITE * 0xFF;
	    }

	    for (uint8_t sec=0; sec < 60; ++sec)
	    {
	    	uint8_t col;				// variable with color of pixel
	    	if (time.seconds > sec)
	    	{
	    		col = COLOR_BLACK;
	    	}
	    	else
	    	{
	    		col = COLOR_WHITE;
	    	}
	    	canvas_setpixel(&image_buffer, 0, EPD_HEIGHT / 2 + 30 - sec, col);
	    	canvas_setpixel(&image_buffer, 1, EPD_HEIGHT / 2 + 30 - sec, col);
	    	canvas_setpixel(&image_buffer, 2, EPD_HEIGHT / 2 + 30 - sec, col);
	    	canvas_setpixel(&image_buffer, 3, EPD_HEIGHT / 2 + 30 - sec, col);
	    }
	   epd_set_frame(&image_buffer, 170, 0, image_buffer.width, image_buffer.height);
		#endif

#ifdef TEST_TEXT
	   // canvas_display_text(&image_buffer,&font24, "Hello world!", 48, 0, 1);

#endif


	    epd_display_frame();
	    epd_deep_sleep();
	    display_refresh_needed = 0;

	    while (!display_refresh_needed)		// loop until display refresh needed
	    {
	    	sleep_mode();			// Enter sleep mode - IDLE mode

//	    	// if connected to charger then enable backlight
//	    	DDRC &= (1<< DDC0);
//	    	if (PINC & (1<<PINC0))
//	    	{
//	    		PORTD |= (1 << PORTD6);
//	    	}
//	    	else
//	    	{
//	    		PORTD &= ~(1<< PORTD6);
//	    	}


	    	// disable backlight when time is out
	    	if (sw_timer[SW_TIMER_BACKLIGHT] == 0)
	    	{
	    		PORTD &= ~(1<<PORTD6);
	    	}

	    	// Button 1 - set time
	    	if (button1_state())
	    	{
	    		app_set_time(&time);
	    		display_refresh_needed = 1;
	    	}

	    	// Button 2 - status screen
	    	if (button2_state())
	    	{
	    		app_status_screen();
	    		display_refresh_needed = 1;
	    	}

	    	// Button 3 - refresh screen
	    	if(button3_state())
	    	{
	    		epd_reset();
	    		epd_init_full(DISPLAY_TEMPERTURE);
	    		epd_clear_frame_memory(COLOR_WHITE);
	    	    epd_display_frame();
	    		epd_clear_frame_memory(COLOR_WHITE);		// todo: second cleaning should be not needed, but when changing apps the memory is not cleared
	    	    epd_display_frame();
	    		display_refresh_needed = 1;
	    	}

	    	// Button 4 - turn on backlight
	    	if(button4_state())
	    	{
	    		PORTD |= (1<<PORTD6);
	    		sw_timer[SW_TIMER_BACKLIGHT] = BACKLIGHT_TIME;
	    	}
	    }
	}
}

ISR(TIMER2_OVF_vect)
{
	for (uint8_t i = 0; i < SW_TIMER_COUNT; ++i)
	{
		if (sw_timer[i] > 0)
			{
			--sw_timer[i];
			}
	}

	++time.seconds;
	if (time.seconds > 59)
	{
		time.seconds = 0;
		++time.minutes;
		display_refresh_needed = 1;
		if (time.minutes > 59)
		{
			time.minutes = 0;
			++time.hours;
			if (time.hours > 23)
			{
				time.hours = 0;

			}
		}
	}

	#ifdef REFRESH_1HZ
		display_refresh_needed = 1;
	#endif
}

ISR (PCINT1_vect)
{
}

ISR (PCINT2_vect)
{
}

